package com.yihu.hos.web.framework.util.controller;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.yihu.hos.core.http.HTTPResponse;
import com.yihu.hos.core.http.HttpClientKit;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.constant.BizObject;
import com.yihu.hos.web.framework.constant.ErrorCode;
import com.yihu.hos.web.framework.constant.PageArg;
import com.yihu.hos.web.framework.exception.EsbApiException;
import com.yihu.hos.web.framework.model.DetailModelResult;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.util.DateTimeUtil;
import com.yihu.hos.web.framework.util.DateUtil;
import com.yihu.hos.web.framework.util.SignVerifyUtil;
import com.yihu.hos.web.framework.util.id.ObjectId;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author HZY
 * @vsrsion 1.0
 * Created at 2016/8/8.
 */
@Component
public class BaseController extends AbstractController {
    static private final Logger logger = LoggerFactory.getLogger(BaseController.class);
    private static String ERR_SYSREM_DES = "系统错误,请联系管理员!";

    //    @Value("${deploy.region}")
    Short deployRegion = 3502;

    protected final static String ResourceCount = "X-Total-Count";
    protected final static String ResourceLink = "Link";

//    @Value("${service-gateway.url}")
//    public String url;

    @Autowired
    protected ObjectMapper objectMapper;

    public BaseController() {
    }

    public Envelop getEnvelop(String json){
        try {
            return objectMapper.readValue(json,Envelop.class);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }



    /**
     *将envelop中的DetailList串转化为模板对象集合
     *Envelop envelop = objectMapper.readValue(resultStr,Envelop.class)
     * modelList = envelop.getDetailModelList()
     * @param modelList
     * @param targets
     * @param targetCls
     * @param <T>
     * @return
     */
    public <T> Collection<T> getEnvelopList(List modelList, Collection<T> targets, Class<T> targetCls) {
        try {
            for (Object aModelList : modelList) {
                String objJsonData = objectMapper.writeValueAsString(aModelList);
                T model = objectMapper.readValue(objJsonData, targetCls);
                targets.add(model);
            }
            return targets;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }

    protected ModelAndView handleRequestInternal(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        return null;
    }

    protected Envelop failedSystem() {
        return failed(ERR_SYSREM_DES);
    }

    public Envelop getResult(List detaiModelList, int totalCount, int currPage, int rows) {
        Envelop result = new Envelop();
        result.setSuccessFlg(true);
        result.setDetailModelList(detaiModelList);
        result.setTotalCount(totalCount);
        result.setCurrPage(currPage);
        result.setPageSize(rows);
        if (result.getTotalCount() % result.getPageSize() > 0) {
            result.setTotalPage(result.getTotalCount() / result.getPageSize() + 1);
        } else {
            result.setTotalPage(result.getTotalCount() / result.getPageSize());
        }

        return result;
    }

    public DetailModelResult getDetaiResult(List detaiModelList, int totalCount, int currPage, int rows) {
        DetailModelResult result = new DetailModelResult();
        result.setSuccessFlg(true);
        result.setDetailModelList(detaiModelList);
        result.setTotalCount(totalCount);
        result.setCurrPage(currPage);
        result.setPageSize(rows);
        if (result.getTotalCount() % result.getPageSize() > 0) {
            result.setTotalPage(result.getTotalCount() / result.getPageSize() + 1);
        } else {
            result.setTotalPage(result.getTotalCount() / result.getPageSize());
        }

        return result;
    }




    public String encodeStr(String str) {
        try {
            return str == null ? null : new String(str.getBytes("ISO-8859-1"), "UTF-8");
        } catch (UnsupportedEncodingException var3) {
            logger.error(var3.getMessage());
            return null;
        }
    }

    public List jsonArrayToList(String jsonArray, Class tClass) {
        ArrayList list = new ArrayList();
        ArrayNode newJsonArray = null;
        try {
            newJsonArray = objectMapper.readValue(jsonArray, ArrayNode.class);
            if (newJsonArray != null && newJsonArray.size() > 0) {
                Iterator it = newJsonArray.iterator();
                while (it.hasNext()) {
                    JsonNode jsonObj = (JsonNode) it.next();
                    Object model = objectMapper.readValue(jsonObj.toString(), tClass);
                    list.add(model);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return list;
    }

    public Object jsonToObject(String json, Class tClass) {
        try {
            return objectMapper.readValue(json, tClass);
        } catch (IOException e) {
            e.printStackTrace();

            logger.error(e.getMessage());
        }

        return null;
    }




    public Map<String, String> genRequestParams(String clientId, String api, String secret, Map<String, String> params) {
        Map<String, String> requestBody = new HashMap<>();
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
            String paramsStr = objectMapper.writeValueAsString(params);
            requestBody.put("api", api);
            requestBody.put("param", paramsStr);
            requestBody.put("timestamp", format.format(new Date()));
            requestBody.put("v", "1.0");
            requestBody.put("appKey", clientId);

            SignVerifyUtil paramSign = new SignVerifyUtil();
            paramSign.addParam(requestBody);
//            paramSign.genParam();
            //TODO 获取app secret传入验证
            String md5Sign = paramSign.signParam(secret);
            requestBody.put("sign", md5Sign);
            return requestBody;
        } catch (IOException e) {
            System.out.println("token验证异常：");
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public Envelop failed(String errMsg) {
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(false);
        envelop.setErrorMsg(errMsg);
        return envelop;
    }

    public Envelop success(Object object) {
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(true);
        envelop.setObj(object);
        return envelop;
    }

    /**
     * 将实体集合转换为模型集合。
     *
     * @param sources
     * @param targets
     * @param properties
     * @param <T>
     * @return
     */
    public <T> Collection<T> convertToModels(Collection sources, Collection<T> targets, Class<T> targetCls, String properties) {
        if (sources == null) {
            return null;
        }
        Iterator iterator = sources.iterator();
        while (iterator.hasNext()) {
            Object source = iterator.next();

            T target = (T) BeanUtils.instantiate(targetCls);
            BeanUtils.copyProperties(source, target, propertyDiffer(StringUtils.isEmpty(properties) ? null : properties.split(","), targetCls));
            targets.add(target);
        }

        return targets;
    }

    /**
     * 计算不在类中的属性。
     *
     * @return
     */
    protected String[] propertyDiffer(String[] properties, Class targetCls) {
        if (properties == null || properties.length == 0) return null;

        PropertyDescriptor[] targetPds = BeanUtils.getPropertyDescriptors(targetCls);
        List<String> propertiesList = Arrays.asList(properties);
        List<String> arrayList = new ArrayList<>();

        for (PropertyDescriptor targetPd : targetPds) {
            Method writeMethod = targetPd.getWriteMethod();
            if (writeMethod != null && !propertiesList.contains(targetPd.getName())) {
                arrayList.add(targetPd.getName());
            }
        }

        return arrayList.toArray(new String[arrayList.size()]);
    }


    /**
     * 客户端调用REST接口时，若返回的是分页结果，则需要在响应头中添加资源的总数量及其他资源的分页导航。
     * EHR平台使用响应头中的 X-Total-Count 字段记录资源的总数量，link header作为其他资源的分页导航。
     *
     * @return
     */
    public void pagedResponse(
            HttpServletRequest request,
            HttpServletResponse response,
            Long resourceCount, Integer currentPage, Integer pageSize) {
        if (request == null || response == null) return;

        response.setHeader(ResourceCount, Long.toString(resourceCount));
        if (resourceCount == 0) return;

        if (currentPage == null) currentPage = new Integer(PageArg.DefaultPage);
        if (pageSize == null) pageSize = new Integer(PageArg.DefaultSize);


        String baseUri = "<" + request.getRequestURL().append("?").toString() + request.getQueryString() + ">";
        long firstPage = currentPage == 1 ? -1 : 1;
        long prevPage = currentPage == 1 ? -1 : currentPage - 1;

        long lastPage = resourceCount % pageSize == 0 ? resourceCount / pageSize : resourceCount / pageSize + 1;
        long nextPage = currentPage == lastPage ? -1 : currentPage + 1;

        lastPage = currentPage == lastPage ? -1 : lastPage;

        Map<String, String> map = new HashMap<>();
        if (firstPage != -1)
            map.put("rel='first',", baseUri.replaceAll("page=\\d+", "page=" + Long.toString(firstPage)));
        if (prevPage != -1) map.put("rel='prev',", baseUri.replaceAll("page=\\d+", "page=" + Long.toString(prevPage)));
        if (nextPage != -1) map.put("rel='next',", baseUri.replaceAll("page=\\d+", "page=" + Long.toString(nextPage)));
        if (lastPage != -1) map.put("rel='last',", baseUri.replaceAll("page=\\d+", "page=" + Long.toString(lastPage)));

        response.setHeader(ResourceLink, linkMap(map));
    }

    private String linkMap(Map<String, String> map) {
        StringBuffer links = new StringBuffer("");
        for (String key : map.keySet()) {
            links.append(map.get(key)).append("; ").append(key);
        }

        return links.toString();
    }

    /**
     * 将实体转换为模型。
     *
     * @param source
     * @param targetCls
     * @param properties
     * @param <T>
     * @return
     */
    public <T> T convertToModel(Object source, Class<T> targetCls, String... properties) {
        if (source == null) {
            return null;
        }
        T target = BeanUtils.instantiate(targetCls);
        BeanUtils.copyProperties(source, target, propertyDiffer(properties, targetCls));
        return target;
    }

    public <T> T toEntity(String json, Class<T> entityCls) throws Exception {
        objectMapper.setDateFormat(new SimpleDateFormat(DateTimeUtil.ISO8601Pattern));
        T entity = objectMapper.readValue(json, entityCls);
        return entity;
    }

    public <T> T toEntity(String json, Class<T> entityCls,String datePattern) throws Exception {
        objectMapper.setDateFormat(new SimpleDateFormat(datePattern));
        T entity = objectMapper.readValue(json, entityCls);
        return entity;
    }

    public String toJson(Object data){
        try {
            String json = objectMapper.writeValueAsString(data);
            return json;
        }catch (Exception e){
            e.printStackTrace();
            return null;
        }
    }

    protected int getTotalCount(ResponseEntity responseEntity) {
        return Integer.parseInt(responseEntity.getHeaders().get("X-Total-Count").get(0));
    }

    protected int getTotalCount(HttpServletResponse response) {
        return Integer.parseInt(response.getHeader("X-Total-Count"));
    }

    protected Map putAll(String sourceJson, Map target){
        if(!org.springframework.util.StringUtils.isEmpty(sourceJson)){
            Map<String, String> source = new HashMap<>();
            try {
                source = objectMapper.readValue(sourceJson, Map.class);
            } catch (IOException e) {
                e.printStackTrace();
            }
            for(Object key : source.keySet()){
                target.put(key, source.get(key));
            }
        }
        return target;
    }

    protected String getObjectId(BizObject bizObject){
        return new ObjectId(deployRegion, bizObject).toString();
    }

    protected EsbApiException errMissParm(String msg){
        return new EsbApiException(ErrorCode.MissParameter, msg);
    }

    protected EsbApiException errNotFound(String objName, Object id){
        return new EsbApiException(ErrorCode.NotFoundObj, objName, String.valueOf(id));
    }

    protected <T> T getModel(Object o){
        return (T) convertToModel(o, getModelClass());
    }

    public Class getModelClass() {
        Type genType = this.getClass().getGenericSuperclass();
        Type[] parameters = ((ParameterizedType) genType).getActualTypeArguments();
        return (Class) parameters[0];
    }

    /**
     * 将日期转为字符串
     *
     * @param dateTime   日期
     * @param formatRule 转换格式
     * @return 日期字符串
     */
    public String DateToString(Date dateTime, String formatRule) {
        try {
            return dateTime == null ? null : DateTimeUtil.utcDateTimeFormat(dateTime);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    /**
     * 将字符串转为Data
     *
     * @param dateTime   日期字符串
     * @param formatRule 转换格式
     * @return 时间格式的日期
     */
    public Date StringToDate(String dateTime, String formatRule) {
        try {
            return dateTime == null ? null : DateUtil.strToDate(dateTime);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    public String trimEnd(String param, String trimChars) {
        if (param.endsWith(trimChars)) {
            param = param.substring(0, param.length() - trimChars.length());
        }
        return param;
    }

}